# -*- coding: utf-8 -*-
import os
import re
import sys
import time
import gzip
import shutil
import random
import urllib
import urllib2

try:
    import json
except ImportError:
    import simplejson as json
import markdown
import markdown.extensions.tables  # for py2exe

import trans
import utils

#~ if sys.platform in ['win32', 'cygwin']:
    #~ PADS_DIR = os.path.join(os.path.abspath(
        #~ utils.module_path()), 'pads.zenpad')
#~ else:
    #~ PADS_DIR = os.path.expanduser('~/pads.zenpad')
PADS_DIR = os.path.expanduser('~/pads.zenpad')
if __name__ == '__main__':
    PADS_DIR = 'var/tests/pad_class/root'  # for tests
AUTO_SLUG = 'new-page'
SYNC_URL = 'http://zenpad.ru/sync%s'
if len(sys.argv) == 2 and sys.argv[1] == 'test':
    SYNC_URL = 'http://localhost:8000/sync%s'


class PageNotFound(Exception): pass
class DirExists(Exception): pass
class BadPagePath(Exception): pass
class CanNotBeMoved(Exception): pass
class SyncError(Exception): pass

HTML_CONVERTERS = (
    ('markdown', lambda text: markdown.markdown(text, ['tables'])),
    ('plain text', utils.plaintext2html),
)


class Page(object):
    DIR_RE = re.compile(r'^[0-9A-Za-z\-]+$')
    PATH_RE = re.compile(r'^/([0-9A-Za-z\-]+/)*$')

    def __init__(self, root, path, content='', encoding='utf-8', gz=False):
        self.root = os.path.abspath(root)
        if self.is_avalible_path(path):
            self.path = path
        else:
            raise BadPagePath(u'bad page path: %s' % path)
        self.content = content
        self.encoding = encoding
        self.gz = gz
        self.file_ext = 'txt.gz' if self.gz else 'txt'
        self._title = None

    @classmethod
    def load(cls, *args, **kwargs):
        page = cls(*args, **kwargs)
        page._content = None
        if not os.path.isfile(page.filename):
            if page.path == '/':
                # create root page
                page.content = ''
            elif page.has_childs():
                # childs dir is exists, create default page
                page.content = ''
            else:
                raise PageNotFound(u'page not found: %s' % page.path)
            page.save()
        return page

    def save(self):
        if os.path.dirname(self.filename) == os.path.dirname(PADS_DIR):
            # HACK: not save beyond PADS_DIR
            return
        if (self.basename.startswith(AUTO_SLUG) and not self.has_childs()
                and (not os.path.exists(self.filename)
                    or not os.path.getsize(self.filename))):
            # first save auto-page
            try:
                index = self.load_config().__order__.index(self.basename)
            except ValueError:
                index = None
            if self.set_basename_by_title() and index is not None:
                cfg = self.load_config()
                cfg.__order__.insert(index, self.basename)
                cfg.save()
        dirname = os.path.dirname(self.filename)
        if not os.path.exists(dirname):
            os.makedirs(dirname)
        opener = gzip.open if self.gz else open
        file = opener(self.filename, 'w')
        file.write(self.content.encode(self.encoding))
        file.close()
        cfg = self.load_config()
        if self.basename not in cfg.__order__:
            cfg.__order__.append(self.basename)
            cfg.save()

    def set_basename_by_title(self):
        ''' 
            >>> root = 'var/tests/page_class/root'
            >>> p = Page(root, '/%s/' % AUTO_SLUG)
            >>> p.content = u'Foo'
            >>> p.save()
            >>> p.basename
            u'foo'
            >>> shutil.rmtree(os.path.dirname(root), ignore_errors=True)
        '''
        if self.path != '/' and self.content.strip():
            basename = name = ''.join([ch if self.DIR_RE.match(ch) else '-'
                for ch in self.title.encode('trans').lower()])
            if basename:
                self.delete()
                i = 1
                dir_list = self.dir_list[:-1]
                while 1:
                    self.path = self.path_as_str(dir_list + [basename])
                    if not os.path.exists(self.filename):
                        break
                    basename = '%s-%s' % (name, i)
                    i += 1
                return True
            
    def delete(self):
        if os.path.exists(self.filename):
            os.unlink(self.filename)
            if self.has_childs():
                shutil.rmtree(self.childs_dirname)
        cfg = self.load_config()
        cfg.pop(self.basename, None)
        if self.basename in cfg.__order__:
            cfg.__order__.remove(self.basename)
        cfg.save()

    @property
    def slug(self):
        return self.dir_list[-1]

    @property
    def basename(self):
        return self.dir_list[-1] or str(os.path.basename(self.root))

    @property
    def content(self):
        if self._content is None:
            file = gzip.open(self.filename) if self.gz else open(self.filename)
            self._content = file.read().decode(self.encoding)
            file.close()
        return self._content
        
    @content.setter
    def content(self, value):
        self._content = unicode(value)
    
    def content_html(self, converter=None):
        return dict(HTML_CONVERTERS)[converter](self.content)

    @property
    def title(self):
        ''' 
            >>> root = 'var/tests/page_class/root'
            >>> p = Page(root, '/')
            >>> p.save()
            >>> p = Page.load(root, '/')
            >>> p.title
            'root'
            >>> p.content = '\t# Title'
            >>> p.save()
            >>> p = Page.load(root, '/')
            >>> p.title
            u'Title'
            >>> shutil.rmtree(os.path.dirname(root), ignore_errors=True)
        '''
        if self._content is not None:
            title = self._content.lstrip(' #\n\r\t').split('\n', 1)[0].rstrip()
        else:
            if self._title is None:
                opener = gzip.open if self.gz else open
                file = opener(self.filename)
                for row in file:
                    row = row.lstrip(' #\n\r\t').rstrip()
                    if row:
                        self._title = row.decode(self.encoding)
                        break
                file.close()
            title = self._title
        if not title:
            title = self.basename
        return title

    @property
    def filename(self):
        return '%s.%s' % (
            os.path.join(self.root, *self.dir_list[1:]), self.file_ext)

    @property
    def childs_dirname(self):
        return self.filename[:-len(self.file_ext) - 1]
    
    def has_childs(self):
        return os.path.exists(self.childs_dirname)

    @property
    def dir_list(self):
        return self.path_as_list(self.path)
    
    @property
    def index(self):
        cfg = self.load_config()
        if self.basename in cfg.__order__:
            return cfg.__order__.index(self.basename)

    @staticmethod
    def is_avalible_dir(dir):
        '''
        Alnum and dash only
            >>> [Page.is_avalible_dir(d) for d in ['', ' ', 'a_b', 'aaa-111']]
            [False, False, False, True]
        '''
        return bool(Page.DIR_RE.match(dir))

    @staticmethod
    def is_avalible_path(path):
        '''
            >>> [Page.is_avalible_path(p) for p in [
            ...     '', '//', 'aa', '/aa', 'aa/', '/a_b/']]
            [False, False, False, False, False, False]
            
            >>> [Page.is_avalible_path(p) for p in ['/', '/a-b/1-2-3/']]
            [True, True]
        '''
        return bool(Page.PATH_RE.match(path))

    @staticmethod
    def path_as_list(path_str):
        '''
        >>> Page.path_as_list('/')
        ['']
        >>> Page.path_as_list('/aaa/bbb/')
        ['', 'aaa', 'bbb']
        '''
        return path_str.rstrip('/').split('/')

    @staticmethod
    def path_as_str(path_list):
        '''
        >>> for path in ['/', '/aaa/', '/aaa/bbb/1-2-3/']:
        ...     dir_list = Page.path_as_list(path)
        ...     Page.path_as_str(dir_list) == path
        True
        True
        True
        '''
        return '/'.join(path_list) + '/'

    def move_to(self, path, index=None):
        '''
            >>> root = 'var/tests/page_class/root'
            >>> Page(root, '/aaa/').save()
            >>> Page(root, '/bbb/ccc/').save()
            >>> p = Page(root, '/aaa/ddd/')
            >>> p.save()
            >>> p.move_to('/bbb/eee/', 0)
            >>> [p.basename for p  in Page.load(root, '/aaa/').get_childs()]
            []
            >>> [p.basename for p  in Page.load(root, '/bbb/').get_childs()]
            ['eee', 'ccc']
            >>> shutil.rmtree(os.path.dirname(root), ignore_errors=True)
        '''
        if not self.is_avalible_path(path):
            raise BadPagePath(u'bad page path: %s' % path)
        if self.path == '/':
            raise CanNotBeMoved
        if path.startswith(self.path) and path != self.path:
            # move to subtree
            raise CanNotBeMoved
        page = Page(self.root, path, encoding=self.encoding, gz=self.gz)
        new_parent = page.get_parent()
        siblings_basenames = [p.basename for p in new_parent.get_childs()]
        if page.basename in siblings_basenames and path != self.path:
            raise DirExists
        if self.path != path:
            # move files
            new_dir = new_parent.childs_dirname
            if not new_parent.has_childs():
                os.makedirs(new_dir)
            if self.has_childs():
                shutil.move(self.childs_dirname, os.path.join(
                    new_dir, page.basename))
            shutil.move(self.filename, os.path.join(
                new_dir, '%s.%s' % (page.basename, page.file_ext)))
        # src config
        cfg = self.load_config()
        page_cfg = cfg.pop(self.basename, None)
        if page_cfg or self.basename in cfg.__order__:
            if self.basename in cfg.__order__:
                cfg.__order__.remove(self.basename)
            cfg.save()
            
        # dst config
        cfg = page.load_config()
        index = index if index is not None else len(cfg.__order__)
        cfg.__order__.insert(index, page.basename)
        if page_cfg:
            cfg[page.basename] = page_cfg
        cfg.save()
        
        self.path = path
    
    def move_up(self):
        '''
            >>> root = 'var/tests/page_class/root'
            >>> Page(root, '/aaa/').save()
            >>> Page(root, '/bbb/').save()
            >>> [p.basename for p  in Page.load(root, '/').get_childs()]
            ['aaa', 'bbb']
            >>> Page.load(root, '/bbb/').move_up()
            >>> [p.basename for p  in Page.load(root, '/').get_childs()]
            ['bbb', 'aaa']
            >>> shutil.rmtree(os.path.dirname(root), ignore_errors=True)
        '''
        index = self.index
        if index:
            self.move_to(self.path, index - 1)

    def move_down(self):
        '''
            >>> root = 'var/tests/page_class/root'
            >>> Page(root, '/aaa/').save()
            >>> Page(root, '/bbb/').save()
            >>> [p.basename for p  in Page.load(root, '/').get_childs()]
            ['aaa', 'bbb']
            >>> Page.load(root, '/aaa/').move_down()
            >>> [p.basename for p  in Page.load(root, '/').get_childs()]
            ['bbb', 'aaa']
            >>> shutil.rmtree(os.path.dirname(root), ignore_errors=True)
        '''
        cfg = self.load_config()
        order = cfg.__order__
        index = order.index(self.basename)
        if index + 1 < len(order):
            self.move_to(self.path, index + 1)

    def move_left(self):
        '''
            >>> root = 'var/tests/page_class/root'
            >>> Page(root, '/aaa/').save()
            >>> Page(root, '/bbb/').save()
            >>> Page(root, '/bbb/ccc/').save()
            >>> Page(root, '/ddd/').save()
            >>> [p.basename for p  in Page.load(root, '/').get_childs()]
            ['aaa', 'bbb', 'ddd']
            >>> Page.load(root, '/bbb/ccc/').move_left()
            >>> [p.basename for p  in Page.load(root, '/').get_childs()]
            ['aaa', 'bbb', 'ccc', 'ddd']
            >>> shutil.rmtree(os.path.dirname(root), ignore_errors=True)
        '''
        parent = self.get_parent()
        new_parent = parent.get_parent()
        if new_parent:
            path = self.path_as_str(new_parent.dir_list + [self.basename])
            self.move_to(path, parent.index + 1)
    
    def move_right(self):
        '''
            >>> root = 'var/tests/page_class/root'
            >>> Page(root, '/aaa/').save()
            >>> Page(root, '/aaa/bbb/').save()
            >>> Page(root, '/ccc/').save()
            >>> [p.basename for p  in Page.load(root, '/aaa/').get_childs()]
            ['bbb']
            >>> Page.load(root, '/ccc/').move_right()
            >>> [p.basename for p  in Page.load(root, '/aaa/').get_childs()]
            ['bbb', 'ccc']
            >>> shutil.rmtree(os.path.dirname(root), ignore_errors=True)
        '''
        cfg = self.load_config()
        order = cfg.__order__
        index = order.index(self.basename)
        if index:
            new_parent_name = order[index - 1]
            path = self.path_as_str(
                self.dir_list[:-1] + [new_parent_name, self.basename])
            self.move_to(path)
        
    def change_basename(self, new_basename):
        '''
            >>> root = 'var/tests/page_class/root'
            >>> Page(root, '/aaa/').save()
            >>> Page(root, '/bbb/').save()
            >>> Page(root, '/ccc/').save()
            >>> [p.basename for p  in Page.load(root, '/').get_childs()]
            ['aaa', 'bbb', 'ccc']
            >>> Page.load(root, '/bbb/').change_basename('ddd')
            >>> [p.basename for p  in Page.load(root, '/').get_childs()]
            ['aaa', 'ddd', 'ccc']
            >>> shutil.rmtree(os.path.dirname(root), ignore_errors=True)
        '''
        if self.basename != new_basename:
            path = self.path_as_str(self.dir_list[:-1] + [new_basename])
            self.move_to(path, self.index)
        
    def load_config(self):
        ext = 'json.gz' if self.gz else 'json'
        filename = os.path.join(
            os.path.dirname(self.filename), 'config.%s' % ext)
        ret = JsonDict(filename, gz=self.gz).load()
        ret.setdefault('__order__', [])
        return ret
    
    def get_parent(self):
        '''
            >>> root = 'var/tests/page_class/root'
            >>> Page(root, '/aaa/').save()
            >>> Page(root, '/aaa/bbb/').save()
            >>> Page.load(root, '/').get_parent()
            >>> Page.load(root, '/aaa/').get_parent().basename
            'root'
            >>> Page.load(root, '/aaa/bbb/').get_parent().basename
            'aaa'
            >>> shutil.rmtree(os.path.dirname(root), ignore_errors=True)
        '''
        if self.path != '/':
            path = self.path_as_str(self.dir_list[:-1])
            return Page.load(self.root, path,
                encoding=self.encoding, gz=self.gz)

    def get_childs(self):
        '''
            >>> root = 'var/tests/page_class/root'
            >>> page = Page(root, '/')
            >>> page.save()
            
        No children:
            >>> page.get_childs()
            []
            >>> os.makedirs(page.childs_dirname)
            >>> page.get_childs()
            []
            
        Order by creation_time dir:
            >>> for i in xrange(5):
            ...     os.makedirs(os.path.join(page.childs_dirname, str(i)))
            ...     time.sleep(0.01)
            >>> [p.basename for p in page.get_childs()]
            ['0', '1', '2', '3', '4']
            >>> Page(root, '/5/').save()
            >>> [p.basename for p in page.get_childs()]
            ['0', '1', '2', '3', '4', '5']
            
        Order by config:
            >>> cfg = page.get_childs()[0].load_config()
            >>> cfg.__order__ = ['5', '4', '3', '2', '1']
            >>> cfg.save()
            >>> [p.basename for p in page.get_childs()]
            ['5', '4', '3', '2', '1', '0']
            
            >>> shutil.rmtree(os.path.dirname(root), ignore_errors=True)
        '''
        pages = []
        childs_dir = self.childs_dirname
        if os.path.isdir(childs_dir):
            # find all nested files and dirs
            items = []
            for name in os.listdir(childs_dir):
                full_name = os.path.join(childs_dir, name)
                if os.path.isfile(full_name):
                    if not name.endswith(self.file_ext):
                        continue
                    name = name[:-len(self.file_ext) - 1]
                elif os.path.isdir(full_name):
                    if os.path.isfile('%s.%s' % (full_name, self.file_ext)):
                        continue
                if not self.is_avalible_dir(name):
                    continue
                items.append((name, os.path.getctime(full_name)))
            if items:
                # sort by creation time
                items.sort(key=lambda x: x[1])
                pages = []
                for name, t in items:
                    path = '%s%s/' % (self.path, name)
                    page = Page.load(self.root, path,
                        encoding=self.encoding, gz=self.gz)
                    pages.append(page)
                # restore saved order
                cfg = pages[0].load_config()
                ordered = cfg.__order__
                items = []
                for page in pages:
                    name = page.basename
                    if name in ordered:
                        items.append((page, ordered.index(name)))
                    else:
                        items.append((page, pages.index(page) + len(ordered)))
                items.sort(key=lambda x: x[1])
                pages = [item[0] for item in items]
                names = [page.basename for page in pages]
                if set(ordered) != set(names):
                    cfg.__order__ = names
                    cfg.save()
        return pages
        
    def __unicode__(self):
        return self.title
    
    def __str__(self):
        return unicode(self).encode('utf-8')


class FilePad(Page):
    def __init__(self, name=None, root=None, **kwargs):
        '''
            >>> pad = FilePad('first')
            >>> pad.basename
            'first'
            >>> shutil.rmtree(os.path.dirname(PADS_DIR), ignore_errors=True)
        '''
        self.name = name
        path = '/%s/' % name if name else '/'
        super(FilePad, self).__init__(root or PADS_DIR, path, **kwargs)
        if name:
            if not os.path.exists(self.pages_root):
                os.makedirs(self.pages_root)
            cfg = FilePad.load_pads_config()
            cfg.__last_used__ = name
            cfg.save()
        self._content = None

    @property
    def pages_root(self):
        return os.path.join(self.root, self.name)

    @classmethod
    def load_pads_config(cls):
        return Page(PADS_DIR, '/aaa/').load_config()
    
    @classmethod
    def all_pads_names(cls):
        '''
            >>> pad = FilePad('first')
            >>> pad = FilePad('second')
            >>> FilePad.all_pads_names()
            ['first', 'second']
            >>> shutil.rmtree(os.path.dirname(PADS_DIR), ignore_errors=True)
        '''
        return [p.basename for p in cls().get_childs()]

    def create_page(self, path, content=''):
        return Page(self.pages_root, path, content,
            encoding=self.encoding, gz=self.gz)
    
    def load_page(self, path):
        '''
            >>> pad = FilePad('first')
            >>> pad.create_page('/aaa/').save()
            >>> pad.create_page('/bbb/').save()
            >>> [p.basename for p in pad.load_page('/').get_childs()]
            ['aaa', 'bbb']
            >>> shutil.rmtree(os.path.dirname(PADS_DIR), ignore_errors=True)
        '''
        return Page.load(self.pages_root, path,
            encoding=self.encoding, gz=self.gz)
    
    def get_free_child_path(self, parent):
        basename = AUTO_SLUG
        i = 1
        while True:
            path = Page.path_as_str(parent.dir_list + [basename])
            try:
                self.load_page(path)
            except PageNotFound:
                break
            basename = '%s-%s' % (AUTO_SLUG, i)
            i += 1
        else:
            raise Exception(u'can not find free child name for page:' % parent.path)
        return path

    def create_auto_child_page(self, page):
        '''
            >>> pad = FilePad('foo')
            >>> page = pad.create_page('/')
            >>> page.save()
            >>> pad.create_auto_child_page(page).save()
            >>> len(page.get_childs())
            1
            >>> shutil.rmtree(os.path.dirname(PADS_DIR), ignore_errors=True)
        '''
        path = self.get_free_child_path(page)
        return self.create_page(path)
        
    def create_auto_sibling_page(self, page):
        '''
            >>> pad = FilePad('foo')
            >>> page = pad.create_page('/aaa/')
            >>> page.save()
            >>> pad.create_auto_sibling_page(page).save()
            >>> len(pad.load_page('/').get_childs())
            2
            >>> shutil.rmtree(os.path.dirname(PADS_DIR), ignore_errors=True)
        '''
        parent = page.get_parent()
        if parent:
            path = self.get_free_child_path(parent)
            sibling = self.create_page(path)
            cfg = page.load_config()
            cfg['__order__'].insert(page.index + 1, sibling.basename)
            cfg.save()
            return sibling
    
    def sync_query(self, uri, **kwargs):
        if 'host' not in kwargs or 'code' not in kwargs:
            pad_cfg = self.load_pads_config().get(self.name)
            kwargs['host'] = kwargs.get('host', pad_cfg.get('host'))
            kwargs['code'] = kwargs.get('code', pad_cfg.get('access_code'))
        while True:
            if not kwargs['host'] or not kwargs['code']:
                data = {'error': 
                    u'Укажите домен и код доступа в настройках блокнота' +
                    u' (получить их можно на zenpad.ru).'}
                break
            try:
                #~ from om.pp import pp
                #~ pp(kwargs)
                kwargs['host'] = kwargs['host'].encode('idna')
                page = urllib2.urlopen(
                    SYNC_URL % uri, urllib.urlencode(kwargs)).read()
            except Exception, ex:
                data = {'error': u'Сервер временно недоступен.'}
                #~ data = {'error': str(ex)}
                break
            try:
                data = json.loads(page)
            except Exception:
                data = {'error': u'Ошибка сервера.'}
                break
            if not isinstance(data, dict):
                data = {'error': u'Ошибка сервера.'}
                break
            break
        if 'error' in data:
            raise SyncError(data['error'])
        return data
    
    def all_pages(self, page=None):
        '''
            >>> pad = FilePad('foo')
            >>> pad.create_page('/aaa/').save()
            >>> pad.create_page('/aaa/bbb/').save()
            >>> pad.create_page('/aaa/ccc/').save()
            >>> [c.path for c in pad.all_pages()]
            ['/', '/aaa/', '/aaa/bbb/', '/aaa/ccc/']
            >>> shutil.rmtree(os.path.dirname(PADS_DIR), ignore_errors=True)
        '''
        page = page or self.load_page('/')
        ret = [page]
        for child in page.get_childs():
            ret.extend(self.all_pages(child))
        return ret

    @property
    def html_converter(self):
        cfg = self.load_pads_config()
        pad_cfg = cfg.setdefault(self.name, {})
        return pad_cfg.get('converter', HTML_CONVERTERS[0][0])

class JsonDict(dict):
    def __init__(self, filename, enc='utf-8', gz=False, *args, **kwargs):
        super(JsonDict, self).__init__(*args, **kwargs)
        self.__dict__['enc'] = enc
        self.__dict__['filename'] = filename
        self.__dict__['gz'] = gz

    def __getattr__(self, name):
        return self[name]

    def __setattr__(self, name, val):
        self[name] = val

    def load(self):
        self.clear()
        opener = gzip.open if self.gz else open
        try:
            file = opener(self.filename)
        except IOError:
            pass
        else:
            self.update(json.loads(file.read().decode(self.enc)))
            file.close()
        return self

    def save(self):
        dir = os.path.dirname(self.filename)
        if not os.path.exists(dir):
            os.makedirs(dir)
        opener = gzip.open if self.gz else open
        json = str(self)
        file = opener(self.filename, 'w')
        file.write(json)
        file.close()
        
    def __str__(self):
        return json.dumps(dict(self), indent=1, sort_keys=True,
            ensure_ascii=False).encode(self.enc)


def main():
    pass


if __name__ == "__main__":
    #~ main()
    #~ return
    
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/mainform.ui'
#
# Created: Thu Sep  8 16:31:03 2011
#      by: PyQt4 UI code generator 4.7.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(689, 559)
        MainWindow.setMinimumSize(QtCore.QSize(100, 100))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/main.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setDocumentMode(False)
        MainWindow.setDockNestingEnabled(False)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_2 = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout_2.setSpacing(1)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 1)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tabWidget = QtGui.QTabWidget(self.centralwidget)
        self.tabWidget.setMinimumSize(QtCore.QSize(100, 100))
        self.tabWidget.setUsesScrollButtons(True)
        self.tabWidget.setDocumentMode(True)
        self.tabWidget.setTabsClosable(False)
        self.tabWidget.setMovable(False)
        self.tabWidget.setObjectName("tabWidget")
        self.text_tab = QtGui.QWidget()
        self.text_tab.setObjectName("text_tab")
        self.gridLayout_3 = QtGui.QGridLayout(self.text_tab)
        self.gridLayout_3.setMargin(0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.tabWidget.addTab(self.text_tab, "")
        self.html_tab = QtGui.QWidget()
        self.html_tab.setObjectName("html_tab")
        self.gridLayout_5 = QtGui.QGridLayout(self.html_tab)
        self.gridLayout_5.setMargin(0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.webView = QtWebKit.QWebView(self.html_tab)
        self.webView.setUrl(QtCore.QUrl("about:blank"))
        self.webView.setObjectName("webView")
        self.gridLayout_5.addWidget(self.webView, 0, 0, 1, 1)
        self.tabWidget.addTab(self.html_tab, "")
        self.source_tab = QtGui.QWidget()
        self.source_tab.setObjectName("source_tab")
        self.gridLayout_6 = QtGui.QGridLayout(self.source_tab)
        self.gridLayout_6.setMargin(0)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.htmlSource = QtGui.QPlainTextEdit(self.source_tab)
        self.htmlSource.setReadOnly(True)
        self.htmlSource.setObjectName("htmlSource")
        self.gridLayout_6.addWidget(self.htmlSource, 0, 0, 1, 1)
        self.tabWidget.addTab(self.source_tab, "")
        self.gridLayout_2.addWidget(self.tabWidget, 0, 0, 1, 2)
        self.slugEdit = QtGui.QLineEdit(self.centralwidget)
        self.slugEdit.setObjectName("slugEdit")
        self.gridLayout_2.addWidget(self.slugEdit, 1, 1, 1, 1)
        self.slugLabel = QtGui.QLabel(self.centralwidget)
        self.slugLabel.setMargin(3)
        self.slugLabel.setObjectName("slugLabel")
        self.gridLayout_2.addWidget(self.slugLabel, 1, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.dockWidget = QtGui.QDockWidget(MainWindow)
        self.dockWidget.setEnabled(True)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dockWidget.sizePolicy().hasHeightForWidth())
        self.dockWidget.setSizePolicy(sizePolicy)
        self.dockWidget.setMinimumSize(QtCore.QSize(92, 166))
        self.dockWidget.setStyleSheet("None")
        self.dockWidget.setFloating(False)
        self.dockWidget.setFeatures(QtGui.QDockWidget.NoDockWidgetFeatures)
        self.dockWidget.setObjectName("dockWidget")
        self.dockWidgetContents = QtGui.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.gridLayout = QtGui.QGridLayout(self.dockWidgetContents)
        self.gridLayout.setSpacing(3)
        self.gridLayout.setContentsMargins(0, 3, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.padList = QtGui.QComboBox(self.dockWidgetContents)
        self.padList.setFrame(True)
        self.padList.setObjectName("padList")
        self.gridLayout.addWidget(self.padList, 0, 0, 1, 1)
        self.tree = QtGui.QTreeWidget(self.dockWidgetContents)
        self.tree.setAutoFillBackground(False)
        self.tree.setFrameShape(QtGui.QFrame.StyledPanel)
        self.tree.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.tree.setIndentation(20)
        self.tree.setRootIsDecorated(True)
        self.tree.setUniformRowHeights(False)
        self.tree.setItemsExpandable(True)
        self.tree.setAnimated(False)
        self.tree.setWordWrap(False)
        self.tree.setHeaderHidden(True)
        self.tree.setExpandsOnDoubleClick(True)
        self.tree.setObjectName("tree")
        self.tree.headerItem().setText(0, "1")
        self.tree.header().setDefaultSectionSize(0)
        self.tree.header().setMinimumSectionSize(0)
        self.tree.header().setStretchLastSection(True)
        self.gridLayout.addWidget(self.tree, 1, 0, 1, 1)
        self.dockWidget.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget)
        self.toolBar = QtGui.QToolBar(MainWindow)
        self.toolBar.setIconSize(QtCore.QSize(16, 16))
        self.toolBar.setObjectName("toolBar")
        MainWindow.addToolBar(QtCore.Qt.ToolBarArea(QtCore.Qt.TopToolBarArea), self.toolBar)
        self.actionSave = QtGui.QAction(MainWindow)
        self.actionSave.setCheckable(False)
        self.actionSave.setEnabled(False)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSave.setIcon(icon1)
        self.actionSave.setAutoRepeat(True)
        self.actionSave.setObjectName("actionSave")
        self.actionAddChild = QtGui.QAction(MainWindow)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/add_child.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionAddChild.setIcon(icon2)
        self.actionAddChild.setAutoRepeat(True)
        self.actionAddChild.setObjectName("actionAddChild")
        self.actionDelete = QtGui.QAction(MainWindow)
        self.actionDelete.setEnabled(True)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/delete.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionDelete.setIcon(icon3)
        self.actionDelete.setObjectName("actionDelete")
        self.actionMoveLeft = QtGui.QAction(MainWindow)
        self.actionMoveLeft.setEnabled(False)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/left.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionMoveLeft.setIcon(icon4)
        self.actionMoveLeft.setObjectName("actionMoveLeft")
        self.actionMoveRight = QtGui.QAction(MainWindow)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/right.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionMoveRight.setIcon(icon5)
        self.actionMoveRight.setObjectName("actionMoveRight")
        self.actionMoveUp = QtGui.QAction(MainWindow)
        self.actionMoveUp.setEnabled(True)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/up.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionMoveUp.setIcon(icon6)
        self.actionMoveUp.setObjectName("actionMoveUp")
        self.actionMoveDown = QtGui.QAction(MainWindow)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/down.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionMoveDown.setIcon(icon7)
        self.actionMoveDown.setObjectName("actionMoveDown")
        self.actionAddSibling = QtGui.QAction(MainWindow)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/add_sibling.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionAddSibling.setIcon(icon8)
        self.actionAddSibling.setObjectName("actionAddSibling")
        self.actionNewPad = QtGui.QAction(MainWindow)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/new_pad.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionNewPad.setIcon(icon9)
        self.actionNewPad.setObjectName("actionNewPad")
        self.actionSettings = QtGui.QAction(MainWindow)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/settings.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSettings.setIcon(icon10)
        self.actionSettings.setObjectName("actionSettings")
        self.actionSync = QtGui.QAction(MainWindow)
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/sync.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSync.setIcon(icon11)
        self.actionSync.setObjectName("actionSync")
        self.actionCopyUrl = QtGui.QAction(MainWindow)
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/copy_link.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionCopyUrl.setIcon(icon12)
        self.actionCopyUrl.setObjectName("actionCopyUrl")
        self.toolBar.addAction(self.actionNewPad)
        self.toolBar.addAction(self.actionSave)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionSettings)
        self.toolBar.addAction(self.actionSync)
        self.toolBar.addAction(self.actionCopyUrl)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionAddChild)
        self.toolBar.addAction(self.actionAddSibling)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionMoveUp)
        self.toolBar.addAction(self.actionMoveDown)
        self.toolBar.addAction(self.actionMoveLeft)
        self.toolBar.addAction(self.actionMoveRight)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionDelete)

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "zenpad.ru", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.text_tab), QtGui.QApplication.translate("MainWindow", "Text Edit", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.html_tab), QtGui.QApplication.translate("MainWindow", "Html View", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.source_tab), QtGui.QApplication.translate("MainWindow", "Html Source", None, QtGui.QApplication.UnicodeUTF8))
        self.slugLabel.setText(QtGui.QApplication.translate("MainWindow", "Url-name", None, QtGui.QApplication.UnicodeUTF8))
        self.tree.setSortingEnabled(False)
        self.toolBar.setWindowTitle(QtGui.QApplication.translate("MainWindow", "toolBar", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave.setText(QtGui.QApplication.translate("MainWindow", "&Save", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave.setToolTip(QtGui.QApplication.translate("MainWindow", "сохранить", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+S", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAddChild.setText(QtGui.QApplication.translate("MainWindow", "&Add Child", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAddChild.setToolTip(QtGui.QApplication.translate("MainWindow", "добавить потомка", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAddChild.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+A", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDelete.setText(QtGui.QApplication.translate("MainWindow", "&Delete", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDelete.setToolTip(QtGui.QApplication.translate("MainWindow", "удалить", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDelete.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+D", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMoveLeft.setText(QtGui.QApplication.translate("MainWindow", "Move&Left", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMoveLeft.setToolTip(QtGui.QApplication.translate("MainWindow", "передвинуть влево", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMoveRight.setText(QtGui.QApplication.translate("MainWindow", "Move&Right", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMoveRight.setToolTip(QtGui.QApplication.translate("MainWindow", "передвинуть вправо", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMoveUp.setText(QtGui.QApplication.translate("MainWindow", "Move&Up", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMoveUp.setToolTip(QtGui.QApplication.translate("MainWindow", "передвинуть вверх", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMoveDown.setText(QtGui.QApplication.translate("MainWindow", "Move&Down", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMoveDown.setToolTip(QtGui.QApplication.translate("MainWindow", "передвинуть вниз", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAddSibling.setText(QtGui.QApplication.translate("MainWindow", "AddS&ibling", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAddSibling.setToolTip(QtGui.QApplication.translate("MainWindow", "добавить соседа", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNewPad.setText(QtGui.QApplication.translate("MainWindow", "NewPad", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNewPad.setToolTip(QtGui.QApplication.translate("MainWindow", "создать новый блокнот", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSettings.setText(QtGui.QApplication.translate("MainWindow", "Settings", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSettings.setToolTip(QtGui.QApplication.translate("MainWindow", "Настройки", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSync.setText(QtGui.QApplication.translate("MainWindow", "Sync", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSync.setToolTip(QtGui.QApplication.translate("MainWindow", "Синхронизация с zenpad.ru", None, QtGui.QApplication.UnicodeUTF8))
        self.actionCopyUrl.setText(QtGui.QApplication.translate("MainWindow", "getUrl", None, QtGui.QApplication.UnicodeUTF8))
        self.actionCopyUrl.setToolTip(QtGui.QApplication.translate("MainWindow", "Скопировать урл страницы", None, QtGui.QApplication.UnicodeUTF8))

from PyQt4 import QtWebKit
import resources_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/sync.ui'
#
# Created: Thu Sep  8 16:31:03 2011
#      by: PyQt4 UI code generator 4.7.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(476, 363)
        self.verticalLayout = QtGui.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.stage = QtGui.QLabel(Dialog)
        self.stage.setText("")
        self.stage.setAlignment(QtCore.Qt.AlignCenter)
        self.stage.setObjectName("stage")
        self.verticalLayout.addWidget(self.stage)
        self.progress = QtGui.QProgressBar(Dialog)
        self.progress.setProperty("value", 0)
        self.progress.setObjectName("progress")
        self.verticalLayout.addWidget(self.progress)
        self.logEdit = QtGui.QPlainTextEdit(Dialog)
        self.logEdit.setReadOnly(True)
        self.logEdit.setCenterOnScroll(False)
        self.logEdit.setObjectName("logEdit")
        self.verticalLayout.addWidget(self.logEdit)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Синхронизация с zenpad.ru", None, QtGui.QApplication.UnicodeUTF8))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/settings.ui'
#
# Created: Thu Sep  8 16:31:03 2011
#      by: PyQt4 UI code generator 4.7.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(495, 253)
        self.gridLayout = QtGui.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 2, 0, 1, 1)
        self.tabWidget = QtGui.QTabWidget(Dialog)
        self.tabWidget.setObjectName("tabWidget")
        self.pad_tab = QtGui.QWidget()
        self.pad_tab.setObjectName("pad_tab")
        self.groupBox = QtGui.QGroupBox(self.pad_tab)
        self.groupBox.setGeometry(QtCore.QRect(10, 60, 451, 81))
        self.groupBox.setObjectName("groupBox")
        self.label = QtGui.QLabel(self.groupBox)
        self.label.setGeometry(QtCore.QRect(10, 50, 121, 16))
        self.label.setObjectName("label")
        self.label_2 = QtGui.QLabel(self.groupBox)
        self.label_2.setGeometry(QtCore.QRect(10, 20, 121, 16))
        self.label_2.setObjectName("label_2")
        self.AccessCodeEdit = QtGui.QLineEdit(self.groupBox)
        self.AccessCodeEdit.setGeometry(QtCore.QRect(130, 50, 311, 21))
        self.AccessCodeEdit.setObjectName("AccessCodeEdit")
        self.PadHostEdit = QtGui.QLineEdit(self.groupBox)
        self.PadHostEdit.setGeometry(QtCore.QRect(130, 20, 311, 21))
        self.PadHostEdit.setObjectName("PadHostEdit")
        self.converterSelect = QtGui.QComboBox(self.pad_tab)
        self.converterSelect.setGeometry(QtCore.QRect(290, 20, 171, 22))
        self.converterSelect.setObjectName("converterSelect")
        self.label_3 = QtGui.QLabel(self.pad_tab)
        self.label_3.setGeometry(QtCore.QRect(170, 20, 111, 16))
        self.label_3.setObjectName("label_3")
        self.tabWidget.addTab(self.pad_tab, "")
        self.gridLayout.addWidget(self.tabWidget, 0, 0, 1, 1)

        self.retranslateUi(Dialog)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), Dialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Настройки", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("Dialog", "Синхронизация с zenpad.ru", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Dialog", "Код доступа", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Dialog", "Домен блокнота", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("Dialog", "Формат данных", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.pad_tab), QtGui.QApplication.translate("Dialog", "Настройки блокнота", None, QtGui.QApplication.UnicodeUTF8))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/add_pad.ui'
#
# Created: Thu Sep  8 16:31:03 2011
#      by: PyQt4 UI code generator 4.7.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(400, 157)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setGeometry(QtCore.QRect(10, 120, 381, 32))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.lineEdit = QtGui.QLineEdit(Dialog)
        self.lineEdit.setGeometry(QtCore.QRect(238, 60, 131, 21))
        self.lineEdit.setMaxLength(30)
        self.lineEdit.setObjectName("lineEdit")
        self.label_root = QtGui.QLabel(Dialog)
        self.label_root.setGeometry(QtCore.QRect(5, 60, 221, 20))
        self.label_root.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.label_root.setObjectName("label_root")
        self.label_2 = QtGui.QLabel(Dialog)
        self.label_2.setGeometry(QtCore.QRect(380, 60, 21, 20))
        self.label_2.setObjectName("label_2")
        self.label_3 = QtGui.QLabel(Dialog)
        self.label_3.setGeometry(QtCore.QRect(10, 10, 381, 41))
        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.converterSelect = QtGui.QComboBox(Dialog)
        self.converterSelect.setGeometry(QtCore.QRect(240, 90, 131, 22))
        self.converterSelect.setObjectName("converterSelect")
        self.label = QtGui.QLabel(Dialog)
        self.label.setGeometry(QtCore.QRect(116, 90, 111, 20))
        self.label.setObjectName("label")

        self.retranslateUi(Dialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), Dialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Новый блокнот", None, QtGui.QApplication.UnicodeUTF8))
        self.label_root.setText(QtGui.QApplication.translate("Dialog", "TextLabel", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Dialog", "/", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("Dialog", "Укажите папку, в которой будет размещаться блокнот\n"
"(используйте только английские буквы, цифры и тире)", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Dialog", "Формат данных", None, QtGui.QApplication.UnicodeUTF8))




# -*- coding: utf-8 -*-
import os
import sys
import re
import cgi

TXT_HTML_RE = re.compile(r'(?P<htmlchars>[<&>])|(?P<space>^[ \t]+)|(?P<lineend>\r\n|\r|\n)|(?P<protocal>(^|\s)((http|ftp)://.*?))(\s|$)', re.S|re.M|re.I)
def plaintext2html(text, tabstop=4):
    def do_sub(m):
        c = m.groupdict()
        if c['htmlchars']:
            return cgi.escape(c['htmlchars'])
        if c['lineend']:
            return '<br>'
        elif c['space']:
            t = m.group().replace('\t', '&nbsp;'*tabstop)
            t = t.replace(' ', '&nbsp;')
            return t
        elif c['space'] == '\t':
            return ' '*tabstop;
        else:
            url = m.group('protocal')
            if url.startswith(' '):
                prefix = ' '
                url = url[1:]
            else:
                prefix = ''
            last = m.groups()[-1]
            if last in ['\n', '\r', '\r\n']:
                last = '<br>'
            return '%s<a href="%s">%s</a>%s' % (prefix, url, url, last)
    return re.sub(TXT_HTML_RE, do_sub, text)
    

def module_path():
    '''for py2exe'''
    if hasattr(sys, "frozen"):
        return os.path.dirname(
            unicode(sys.executable, sys.getfilesystemencoding( ))
        )
    return os.path.dirname(unicode(__file__, sys.getfilesystemencoding( )))


#! /usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
from binascii import crc32
try:
    from hashlib import md5
except ImportError:
    from md5 import md5
from PyQt4 import QtCore, QtGui
from PyQt4 import QtNetwork  # for py2exe

import filepad
import utils
if len(sys.argv) == 2 and sys.argv[1] == 'update_ui':
    # recompile ui
    for filename in ['mainform', 'add_pad', 'settings', 'sync']:
        os.system('pyuic4 -o ui/%s.py ui/%s.ui' % (filename, filename))
        os.system('pyrcc4 -o ui/resources_rc.py ui/resources.qrc')
import ui


DIR_EXISTS_MSG = u'Там, куда вы передвигаете страницу, уже существует страница с таким же slug. Поменяйте slug одной из страниц.'
CSS = '''<style type="text/css">
h1, h2, h3, h4, h5, h6, npre, blockquote, ul, ol, dl {margin: 20px 0;}  
h1 {font-size: 130%; margin-top: 10px;}
h2 {font-size: 120%; margin-top: 25px;}
h3 {font-size: 110%;}
pre {white-space: pre;}
pre code {background: #f8fff8; display: block; padding: 5px; border: 1px solid #ded; border-left: 3px solid #ded;}
table {border-collapse: collapse; font-size: 100%;}
td, th {padding: 5px 10px;}
th {text-align: center; font-weight: bold; border: 1px solid gray;}
td {text-align: left; border: 1px dashed gray;}
</style>'''
INTRO_PAGE = '''
Добро пожаловать в ZenPad
=========================

- документация по блокноту доступна по адресу: <http://about.zenpad.ru/>
- задать вопрос или сообщить об ошибке можно в группе: <http://groups.google.com/group/zenpad>

Приятной работы :)'''.strip()


class PageTreeItem(QtGui.QTreeWidgetItem):
    def __init__(self, page):
        super(QtGui.QTreeWidgetItem, self).__init__([page.title])
        self.page_slug = page.slug
    
    @property
    def page_path(self):
        items = [self]
        while True:
            parent = items[0].parent()
            if not parent:
                break
            items.insert(0, parent)
        return filepad.Page.path_as_str([item.page_slug for item in items])


class MainForm(QtGui.QMainWindow, ui.mainform.Ui_MainWindow):
    def __init__(self, parent=None):
        super(MainForm, self).__init__(parent)
        self.setupUi(self)
        self.showMaximized()
        
        self.textEdit = TextEdit(self.text_tab)
        self.textEdit.setLineWrapMode(QtGui.QPlainTextEdit.NoWrap)
        self.textEdit.setObjectName("textEdit")
        self.gridLayout_3.addWidget(self.textEdit, 0, 0, 1, 1)
        self.slugEdit.setValidator(QtGui.QRegExpValidator(
            QtCore.QRegExp('([0-9A-Za-z\-])+'), self.slugEdit))
        # hide QDockWidget title
        self.dockWidget.setTitleBarWidget(QtGui.QWidget())

        self.connect(self.tree,
            QtCore.SIGNAL('itemClicked (QTreeWidgetItem *,int)'), self.treeClick)
        self.connect(self.textEdit,
            QtCore.SIGNAL('textChanged ()'), self.textChanged)
        self.connect(self.tabWidget,
            QtCore.SIGNAL('currentChanged (int)'), self.tabChanged)
        self.connect(self.padList,
            QtCore.SIGNAL('activated (const QString&)'), self.padChanged)
        self.connect(self.slugEdit,
            QtCore.SIGNAL('editingFinished ()'), self.slugChanged)
            
        self.init_pad()
        
    def init_pad(self, pad_name=None):
        pad = filepad.FilePad()
        name_list = pad.all_pads_names()
        if not name_list:
            os.makedirs(os.path.join(filepad.PADS_DIR, 'first'))
            open(os.path.join(
                filepad.PADS_DIR, 'first.txt'), 'w').write(INTRO_PAGE)
            name_list.append('first')
        cfg = filepad.FilePad.load_pads_config()
        pad_name = pad_name or cfg.get('__last_used__')
        if pad_name not in name_list:
            pad_name = name_list[0]
        self.padList.clear()
        self.padList.addItems(name_list)
        self.padList.setCurrentIndex(name_list.index(pad_name))
        self.tree.clear()
        self.pad = filepad.FilePad(pad_name)
        last_path = cfg.get(pad_name, {}).get('last_page', '/')
        try:
            cur_page = self.pad.load_page(last_path)
        except filepad.PageNotFound:
            cur_page = self.pad.load_page('/')
        self.load_tree(cur_page.path)
        self.page = None
        self.load_page(cur_page.path)
        active_tab = cfg.get(pad_name, {}).get('active_tab', 1)
        self.tabWidget.setCurrentIndex(active_tab)


    def load_tree(self, cur_page_path):
        root = self.pad.load_page('/')
        item = PageTreeItem(root)
        self.tree.addTopLevelItem(item)
        if cur_page_path == '/':
            self.tree.setCurrentItem(item)
            self.tree.expandItem(item)
        self._load_tree_rec(cur_page_path, root, item)
    
    def _load_tree_rec(self, cur_page_path, parent, parent_item):
        for child in parent.get_childs():
            item = PageTreeItem(child)
            parent_item.addChild(item)
            if item.page_path == cur_page_path:
                self.tree.setCurrentItem(item)
                self.tree.expandItem(item)
            self._load_tree_rec(cur_page_path, child, item)

    def load_page(self, path):
        self.save_cur_page()
        self.page = self.pad.load_page(path)
        self.page.menu_item = self.tree.currentItem()
        self.textEdit.setPlainText(self.page.content)
        self.slugEdit.setText(self.page.basename)
        if self.page.path == '/':
            self.slugLabel.setText(u'Папка блокнота:')
        else:
            self.slugLabel.setText(u'Slug:')
        if self.tabWidget.currentIndex() == 0:
            self.textEdit.setFocus()
        self._update_html()
        self.actionSave.setDisabled(True)
        self._set_button_status()
        self._update_last_used_page()
    
    def _update_last_used_page(self):
        cfg = self.pad.load_pads_config()
        pad_cfg = cfg.setdefault(self.pad.name, {})
        pad_cfg['last_page'] = self.page.path
        cfg.save()
    
    def _update_html(self):
        self.page.content = unicode(self.textEdit.toPlainText())
        if self.tabWidget.currentIndex() == 1:
            self.webView.setHtml( 
                CSS + self.page.content_html(self.pad.html_converter))
        elif self.tabWidget.currentIndex() == 2:
            self.htmlSource.setPlainText(
                self.page.content_html(self.pad.html_converter))
        
    def _set_button_status(self):
        parent = self.page.menu_item.parent()
        index = parent.indexOfChild(self.page.menu_item) if parent else -1
        if parent:
            self.actionAddSibling.setEnabled(True)
            if index:
                self.actionMoveUp.setEnabled(True)
            else:
                self.actionMoveUp.setDisabled(True)
            if index + 1 < parent.childCount():
                self.actionMoveDown.setEnabled(True)
            else:
                self.actionMoveDown.setDisabled(True)
            if parent.parent():
                self.actionMoveLeft.setEnabled(True)
            else:
                self.actionMoveLeft.setDisabled(True)
            if index:
                self.actionMoveRight.setEnabled(True)
            else:
                self.actionMoveRight.setDisabled(True)
        else:
            self.actionMoveUp.setDisabled(True)
            self.actionMoveDown.setDisabled(True)
            self.actionMoveLeft.setDisabled(True)
            self.actionMoveRight.setDisabled(True)
            self.actionAddSibling.setDisabled(True)
    
    def save_cur_page(self):
        if self.page and self.actionSave.isEnabled():
            self.page.content = unicode(self.textEdit.toPlainText())
            self.page.save()
            old_dir = unicode(self.slugEdit.text())
            if old_dir.startswith(filepad.AUTO_SLUG):
                # basename may be changed
                self.page.menu_item.page_slug = self.page.slug
                self.slugEdit.setText(self.page.basename)
                self._update_last_used_page()
            self.actionSave.setDisabled(True)
            self.page.menu_item.setData(
                0, QtCore.Qt.DisplayRole, QtCore.QVariant(self.page.title))
    
    def treeClick(self, item, column):
        self.load_page(item.page_path)

    def textChanged(self):
        if self.page:
            content = unicode(self.textEdit.toPlainText())
            if self.page.content != content:
                self.actionSave.setEnabled(True)
            title = content.lstrip(' #\n\r\t').split('\n', 1)[0].rstrip()
            title = title or self.page.basename
            if unicode(self.page.menu_item.text(0)) != title:
                self.page.menu_item.setText(0, title)

    def tabChanged(self, tab_id):
        if tab_id == 0:
            self.textEdit.setFocus()
        else:
            self._update_html()
        cfg = filepad.FilePad.load_pads_config()
        cfg.setdefault(self.pad.name, {})['active_tab'] = tab_id
        cfg.save()
    
    def padChanged(self, pad_name):
        self.save_cur_page()
        pad_name = unicode(pad_name)
        self.init_pad(pad_name)

    def slugChanged(self):
        slug = unicode(self.slugEdit.text())
        if self.page.path == '/' and self.pad.basename != slug:
            # change pad name
            self.save_cur_page()
            try:
                self.pad.change_basename(slug)
            except filepad.DirExists:
                self.slugEdit.setText(self.pad.name)
                QtGui.QMessageBox.critical(self, u'Ошибка',
                    u'Блокнот с таким именем уже существует',
                QtGui.QMessageBox.Yes)
            else:
                self.init_pad(slug)
        elif self.page.basename != slug:
            # change page slug
            self.save_cur_page()
            try:
                self.page.change_basename(slug)
            except filepad.DirExists:
                self.slugEdit.setText(self.page.basename)
                QtGui.QMessageBox.critical(self, u'Ошибка',
                    u'Такой slug уже существует в этой папке',
                    QtGui.QMessageBox.Yes)
            else:
                self.page.menu_item.page_slug = self.page.slug
                self.page.menu_item.setData(
                    0, QtCore.Qt.DisplayRole, QtCore.QVariant(self.page.title))
                self._update_last_used_page()

    def on_actionCopyUrl_triggered(self, checked=None):
        if checked is not None:
            cfg = self.pad.load_pads_config()
            host = cfg.get(self.pad.name, {}).get('host')
            if not host:
                QtGui.QMessageBox.critical(self, u'Синхронизация не настроена',
                    u'Невозможно сгенерировать url. Укажите домен в настройках синхронизации.',
                    QtGui.QMessageBox.Ok)
                return
            url = u'http://%s%s' % (host, self.page.path)
            clipboard = QtGui.QApplication.clipboard()
            clipboard.setText(url)

    def on_actionSave_triggered(self, checked=None):
        if checked is not None:
            self.save_cur_page()

    def on_actionAddChild_triggered(self, checked=None):
        if checked is None: return
        self.save_cur_page()
        page = self.pad.create_auto_child_page(self.page)
        page.save()
        self.tree.expandItem(self.tree.currentItem())
        menu_item = PageTreeItem(page)
        self.tree.currentItem().addChild(menu_item)
        self.tree.setCurrentItem(menu_item)
        self.load_page(page.path)
        self.tabWidget.setCurrentIndex(0)

    def on_actionAddSibling_triggered(self, checked=None):
        if checked is None: return
        self.save_cur_page()
        page = self.pad.create_auto_sibling_page(self.page)
        page.save()
        item = PageTreeItem(page)
        parent = self.tree.currentItem().parent()
        index = parent.indexOfChild(self.page.menu_item) + 1
        parent.insertChild(index, item)
        self.tree.setCurrentItem(item)
        self.load_page(page.path)
        self.tabWidget.setCurrentIndex(0)

    def on_actionMoveLeft_triggered(self, checked=None):
        if checked is None: return
        parent = self.page.menu_item.parent()
        p_parent = parent.parent()
        index = p_parent.indexOfChild(parent) + 1
        try:
            self.page.move_left()
        except filepad.DirExists:
            QtGui.QMessageBox.critical(self, u'Ошибка',
                DIR_EXISTS_MSG, QtGui.QMessageBox.Yes)
            return
        item = parent.takeChild(parent.indexOfChild(self.page.menu_item))
        item.page_slug = self.page.slug
        p_parent.insertChild(index, item)
        self.tree.setCurrentItem(item)
        self._set_button_status()

    def on_actionMoveRight_triggered(self, checked=None):
        if checked is None: return
        parent = self.page.menu_item.parent()
        index = parent.indexOfChild(self.page.menu_item)
        if index:
            try:
                self.page.move_right()
            except filepad.DirExists:
                QtGui.QMessageBox.critical(self, u'Ошибка',
                    DIR_EXISTS_MSG, QtGui.QMessageBox.Yes)
                return
            new_parent = parent.child(index - 1)
            item = parent.takeChild(index)
            item.page_slug = self.page.slug
            new_parent.addChild(item)
            self.tree.setCurrentItem(item)
            self._set_button_status()

    def on_actionMoveUp_triggered(self, checked=None):
        if checked is None: return
        self.page.move_up()
        parent = self.page.menu_item.parent()
        index = parent.indexOfChild(self.page.menu_item)
        item = parent.takeChild(index)
        parent.insertChild(index-1, item)
        self.tree.setCurrentItem(item)
        self._set_button_status()

    def on_actionMoveDown_triggered(self, checked=None):
        if checked is None: return
        self.page.move_down()
        parent = self.page.menu_item.parent()
        index = parent.indexOfChild(self.page.menu_item)
        item = parent.takeChild(index)
        parent.insertChild(index+1, item)
        self.tree.setCurrentItem(item)
        self._set_button_status()

    def on_actionDelete_triggered(self, checked=None):
        if checked is None: return
        if self.page.path == '/':
            # delete pad
            reply = QtGui.QMessageBox.warning(self, u'Удаление блокнота',
                u'Вы собираетесь удалить весь блокнот! Вы уверены?',
                QtGui.QMessageBox.Yes | QtGui.QMessageBox.No, QtGui.QMessageBox.No)
            if reply == QtGui.QMessageBox.Yes:
                self.pad.delete()
                self.init_pad()
        else:
            # delete page
            if self.page.content.strip() or unicode(self.textEdit.toPlainText()
                    ).strip() or self.page.has_childs():
                reply = QtGui.QMessageBox.question(self, u'Удаление',
                    u'Удалить эту страницу и все вложенные?',
                    QtGui.QMessageBox.Yes | QtGui.QMessageBox.No, QtGui.QMessageBox.No)
                if reply != QtGui.QMessageBox.Yes:
                    return
            self.page.delete()
            parent = self.page.menu_item.parent()
            parent.takeChild(parent.indexOfChild(self.page.menu_item))
            self.page = None
            self.load_page(self.tree.currentItem().page_path)

    def closeEvent(self, event):
        #~ reply = QtGui.QMessageBox.question(self, u'Выход',
            #~ u'Хотите выйти?', QtGui.QMessageBox.Yes | 
            #~ QtGui.QMessageBox.No, QtGui.QMessageBox.No)
        #~ if reply != QtGui.QMessageBox.Yes:
            #~ event.ignore()
        self.save_cur_page()
        event.accept()

    def on_actionNewPad_triggered(self, checked=None):
        if checked is None: return
        self.save_cur_page()
        dlg = AddPadForm(self)
        res = dlg.exec_()
        if res:
            os.makedirs(dlg.path)
            self.init_pad(dlg.pad_name)

    def on_actionSettings_triggered(self, checked=None):
        if checked is None: return
        self.save_cur_page()
        SettingsForm(self).exec_()
        self._update_html()

    def on_actionSync_triggered(self, checked=None):
        if checked is None: return
        self.save_cur_page()
        SyncForm(self).exec_()


class SyncForm(QtGui.QDialog, ui.sync.Ui_Dialog):
    STAGES = [
        u'Подготовка',
        u'Удаление страниц',
        u'Обновление страниц',
        u'Сортировка',
    ]
    
    def __init__(self, parent=None):
        super(SyncForm, self).__init__(parent)
        self.setupUi(self)
        self.parent = parent
        self.pad = parent.pad
        self.timer_id = self.startTimer(0)
        self.sync_iter = self.sync()
        self.stage.clear()
        self.buttonBox.rejected.connect(self.on_reject)
        self.buttonBox.buttons()[0].setText(u'Отмена')
    
    def on_reject(self):
        self.close()

    def timerEvent(self, event):
        try:
            self.sync_iter.next()
        except StopIteration:
            self.killTimer(self.timer_id)
            self.buttonBox.buttons()[0].setText(
                u'Синхронизация завершена, закрыть окно')
    
    def next_stage(self, skip=False):
        if not hasattr(self, 'stage_id'):
            self.stage_id = 0
        else:
            self.stage_id += 1
        stage = self.STAGES[self.stage_id]
        self.stage.setText(u'Этап %i из %i: %s' % (
            self.stage_id + 1, len(self.STAGES), stage))
        if not skip:
            self.logEdit.appendHtml(u'<h3>%s</h3><br>' % stage)
        self.update()
    
    def log(self, msg):
        self.logEdit.insertPlainText(msg)
        self.update()
    
    def log_ok(self):
        self.log('ok\n')
    
    def set_progress(self, max):
        self.progress.reset()
        self.progress.setRange(1, max)
        self.update()
    
    def inc_progress(self):
        self.progress.setValue(self.progress.value() + 1)
        self.update()
    
    def sync(self):
        self.next_stage()
        html_converter = self.pad.html_converter.encode('utf-8')
        yield
        try:
            self.log(u'Получение списка страниц с сервера ...')
            yield
            web_pages = self.pad.sync_query('/get-page-list/')['pages']
            self.log_ok()
            self.log(u'Подготовка локального списка страниц ... ')
            yield
            loc_pages = self.pad.all_pages()
            loc_paths = [p.path for p in loc_pages]
            self.log_ok()
            self.log(u'Анализ ...\n')
            yield
            to_del = []
            for path, crc, md in web_pages:
                if path not in loc_paths:
                    to_del.append(path)
            to_del.sort()
            to_del.reverse()
            self.log(u'- страниц для удаления: %i\n' % len(to_del))
            yield
            # update list
            to_up = []
            web_pages = dict((path, (crc, md)) for path, crc, md in web_pages)
            self.set_progress(len(loc_pages))
            for page in loc_pages:
                content = open(page.filename).read()
                loc_crc32 = crc32(html_converter + content)
                loc_md5 = md5(html_converter + content).hexdigest()
                if page.path not in web_pages:
                    to_up.append((page.path, loc_crc32, loc_md5))
                else:
                    web_crc32, web_md5 = web_pages.get(page.path)
                    if (loc_crc32, loc_md5) != (web_crc32, web_md5):
                        to_up.append((page.path, loc_crc32, loc_md5))
                self.inc_progress()
                yield
            self.log(u'- страниц для обновления: %i\n' % len(to_up))
            yield
            self.next_stage(not to_del)
            if to_del:
                # delete
                self.set_progress(len(to_del))
                for path in to_del:
                    self.log(u'del: %s ... ' % path)
                    yield
                    self.pad.sync_query('/del/', path=path)
                    self.inc_progress()
                    self.log_ok()
            self.next_stage(not to_up)
            if to_up:
                # update
                converter = self.pad.html_converter
                self.set_progress(len(to_up))
                for path, loc_crc32, loc_md5 in to_up:
                    page = self.pad.load_page(path)
                    self.log(u'up: %s ... ' % path)
                    yield
                    self.pad.sync_query('/up/',
                        path=path,
                        title=page.title.encode('utf-8'),
                        crc32=loc_crc32,
                        md5=loc_md5,
                        content=page.content_html(converter).encode('utf-8'))
                    self.inc_progress()
                    self.log_ok()
            # order
            self.next_stage()
            self.log(u'запрос сортировки с сервера ...')
            yield
            web_paths = self.pad.sync_query('/get-path-list/')['paths']
            self.log_ok()
            if web_paths != loc_paths:
                if set(web_paths) != set(loc_paths):
                    raise filepad.SyncError(
                        u'Ошибка сихрнонизации: обновление не удалось')
                        
                def get_child_slugs(data, path):
                    indent = path.count('/')
                    return [p.rsplit('/', 2)[1] for p in data
                        if p.startswith(path) and p.count('/') - 1 == indent]
                
                self.set_progress(len(loc_paths)) 
                for path in loc_paths:
                    loc_childs = get_child_slugs(loc_paths, path)
                    web_childs = get_child_slugs(web_paths, path)
                    if loc_childs != web_childs:
                        self.log(u'sort: %s ... ' % path)
                        yield
                        self.pad.sync_query('/order/', path=path, 
                            childs='|'.join(loc_childs))
                        self.log_ok()
                    self.inc_progress()
        except filepad.SyncError, ex:
            self.logEdit.appendHtml('<span style="color:red">%s</span>' % ex[0])
        else:
            self.logEdit.appendHtml('<h3 style="color:green">%s</h3>' % 
                u'Синхронизация завершена успешно.')
        self.update()


class SettingsForm(QtGui.QDialog, ui.settings.Ui_Dialog):
    def __init__(self, parent=None):
        super(SettingsForm, self).__init__(parent)
        self.setupUi(self)
        self.parent = parent
        self.buttonBox.accepted.connect(self.on_accept)
        self.pad = parent.pad
        cfg = self.pad.load_pads_config()
        pad_cfg = cfg.setdefault(self.pad.name, {})
        self.PadHostEdit.setText(pad_cfg.get('host', ''))
        self.AccessCodeEdit.setText(pad_cfg.get('access_code', ''))
        items = [a for a, b in filepad.HTML_CONVERTERS]
        self.converterSelect.addItems(items)
        self.converterSelect.setCurrentIndex(
            items.index(self.pad.html_converter))

    def on_accept(self):
        cfg = self.pad.load_pads_config()
        pad_cfg = cfg.setdefault(self.pad.name, {})
        pad_cfg['host'] = unicode(self.PadHostEdit.text())
        pad_cfg['access_code'] = unicode(self.AccessCodeEdit.text())
        pad_cfg['converter'] = unicode(self.converterSelect.currentText())
        cfg.save()
        self.done(1)


class AddPadForm(QtGui.QDialog, ui.add_pad.Ui_Dialog):
    def __init__(self, parent=None):
        super(AddPadForm, self).__init__(parent)
        self.setupUi(self)
        self.parent = parent
        self.label_root.setText(filepad.PADS_DIR + '/')
        self.lineEdit.setFocus()
        self.buttonBox.accepted.connect(self.on_accept)
        self.lineEdit.setValidator(QtGui.QRegExpValidator(
            QtCore.QRegExp('([0-9A-Za-z\-])+'), self.lineEdit))
        items = [a for a, b in filepad.HTML_CONVERTERS]
        self.converterSelect.addItems(items)

    def on_accept(self):
        self.pad_name = str(self.lineEdit.text())
        if self.pad_name:
            self.path = os.path.join(filepad.PADS_DIR, self.pad_name)
            if os.path.isdir(self.path):
                QtGui.QMessageBox.critical(self.parent, u'Ошибка',
                    u'Папка уже существует.', QtGui.QMessageBox.Yes)
                self.done(0)
            else:
                cfg = filepad.FilePad.load_pads_config()
                pad_cfg = cfg.setdefault(self.pad_name, {})
                pad_cfg['converter'] = unicode(self.converterSelect.currentText())
                cfg.save()
                self.done(1)
        else:
            self.done(0)


class TextEdit(QtGui.QPlainTextEdit):
    '''Editor witch block-indentation and auto-newline-indentaiton'''
    def __init__(self, parent):
        QtGui.QTextEdit.__init__(self, parent)
        font = QtGui.QFont()
        self.setStyleSheet('font: 10pt "DejaVu Sans Mono";')
        
    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Tab:
            self.block_indent()
        elif event.key() == QtCore.Qt.Key_Backtab:
            self.block_unindent()
        elif event.key() == QtCore.Qt.Key_Return:
            self.autoindent_newline()
        else:
            super(TextEdit, self).keyPressEvent(event)

    def autoindent_newline(self):
        tab = '    '
        self.indent_character = ':'

        cursor = self.textCursor()
        text = unicode(cursor.block().text())
        trimmed = text.rstrip()
        current_indent_pos = self._get_indent_position(text)

        cursor.beginEditBlock()

        # Create the new line. There is no need to move to the new block, as
        # the insertBlock will do that automatically
        cursor.insertBlock()

        # Remove any leading whitespace from the current line
        after = unicode(cursor.block().text())
        trimmed_after = after.rstrip()
        pos = after.index(trimmed_after)
        for i in range(pos):
            cursor.deleteChar()

        if self.indent_character and trimmed.endswith(self.indent_character):
            # indent one level
            indent = text[:current_indent_pos] + tab
        else:
            # indent to the same level
            indent = text[:current_indent_pos]
        cursor.insertText(indent)

        cursor.endEditBlock()
        self.ensureCursorVisible()

    def block_indent(self):
        cursor = self.textCursor()

        if not cursor.hasSelection():
            # Insert a tabulator
            self.line_indent(cursor)

        else:
            # Indent every selected line
            sel_blocks = self._get_selected_blocks()

            cursor.clearSelection()
            cursor.beginEditBlock()

            for block in sel_blocks:
                cursor.setPosition(block.position())
                self.line_indent(cursor)

            cursor.endEditBlock()
            self._show_selected_blocks(sel_blocks)

    def block_unindent(self):
        cursor = self.textCursor()

        if not cursor.hasSelection():
            # Unindent current line
            position = cursor.position()
            cursor.beginEditBlock()

            removed = self.line_unindent(cursor)
            position = max(position-removed, 0)

            cursor.endEditBlock()
            cursor.setPosition(position)
            self.setTextCursor(cursor)

        else:
            # Unindent every selected line
            sel_blocks = self._get_selected_blocks()

            cursor.clearSelection()
            cursor.beginEditBlock()

            for block in sel_blocks:
                cursor.setPosition(block.position())
                self.line_unindent(cursor)

            cursor.endEditBlock()
            self._show_selected_blocks(sel_blocks)

    def line_indent(self, cursor):
        tab = '    '
        cursor.insertText(tab)

    def line_unindent(self, cursor):
        """ Unindents the cursor's line. Returns the number of characters 
            removed.
        """
        tab = '    '
        cursor.movePosition(QtGui.QTextCursor.StartOfBlock)
        if unicode(cursor.block().text()).startswith(tab):
            new_text = cursor.block().text()[len(tab):]
            cursor.movePosition(QtGui.QTextCursor.EndOfBlock,
                                QtGui.QTextCursor.KeepAnchor)
            cursor.removeSelectedText()
            cursor.insertText(new_text)
            return len(tab)
        else:
            return 0

    def _get_indent_position(self, line):
        trimmed = line.lstrip()
        if trimmed:
            return len(line) - len(trimmed)
        else:
            # if line is all spaces, treat it as the indent position
            return len(line)

    def _show_selected_blocks(self, selected_blocks):
        """ Assumes contiguous blocks
        """
        cursor = self.textCursor()
        cursor.clearSelection()
        cursor.setPosition(selected_blocks[0].position())
        cursor.movePosition(QtGui.QTextCursor.StartOfBlock)
        cursor.movePosition(QtGui.QTextCursor.NextBlock, 
                            QtGui.QTextCursor.KeepAnchor, len(selected_blocks))
        cursor.movePosition(QtGui.QTextCursor.EndOfBlock, 
                            QtGui.QTextCursor.KeepAnchor)

        self.setTextCursor(cursor)

    def _get_selected_blocks(self):
        cursor = self.textCursor()
        if cursor.position() > cursor.anchor():
            move_op = QtGui.QTextCursor.PreviousBlock
            start_pos = cursor.anchor()
            end_pos = cursor.position()
        else:
            move_op = QtGui.QTextCursor.NextBlock
            start_pos = cursor.position()
            end_pos = cursor.anchor()

        cursor.setPosition(start_pos)
        cursor.movePosition(QtGui.QTextCursor.StartOfBlock)
        blocks = [cursor.block()]

        while cursor.movePosition(QtGui.QTextCursor.NextBlock):
            block = cursor.block()
            if block.position() < end_pos:
                blocks.append(block)

        return blocks


def main():
    app = QtGui.QApplication(sys.argv)
    form = MainForm()
    form.show()
    app.exec_()
   
 
if __name__ == "__main__":
    main()



import os
from setuptools import setup

PACKAGE = 'zenpad'
VERSION = '0.0.5'
README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

if __name__ == '__main__':
    # Compile the list of packages available, because distutils doesn't have
    # an easy way to do this.
    packages, data_files = [], []
    root_dir = os.path.dirname(__file__)
    if root_dir:
        os.chdir(root_dir)

    for dirpath, dirnames, filenames in os.walk(PACKAGE):
        for i, dirname in enumerate(dirnames):
            if dirname in ['.', '..']:
                del dirnames[i]
        if '__init__.py' in filenames:
            pkg = dirpath.replace(os.path.sep, '.')
            if os.path.altsep:
                pkg = pkg.replace(os.path.altsep, '.')
            packages.append(pkg)
        elif filenames:
            prefix = dirpath[len(PACKAGE) + 1:] # Strip package directory + path separator
            for f in filenames:
                data_files.append(os.path.join(prefix, f))


    setup(
        name = PACKAGE,
        version = VERSION,
        description = 'Tree-structured notepad',
        long_description = README,
        author = 'Imbolc',
        author_email = 'imbolc@imbolc.name',
        url = 'http://bitbucket.org/imbolc/zenpad/',
        packages = packages,
        package_data = {'zenpad': data_files},
        entry_points = {
            'gui_scripts': [
                'zenpad = zenpad.zenpad:main',
            ]
        },
        license = "BSD",
        classifiers = [
            "Programming Language :: Python",
            "License :: OSI Approved :: MIT License",
            "Development Status :: 4 - Beta",
            "Environment :: X11 Applications :: Qt",
            "Intended Audience :: End Users/Desktop",
            "Natural Language :: Russian",
            "Topic :: Text Editors",
        ],
    )



# coding: utf-8

ur"""

This module translates national characters into similar sounding
latin characters (transliteration).
At the moment, Czech, Greek, Latvian, Polish, Turkish, Russian, Ukrainian
alphabets are supported (it covers 99% of needs).

  >>> # coding: utf-8
  >>> import trans
  >>> u'Hello World!'.encode('trans')
  u'Hello World!'
  >>> u'Привет, Мир!'.encode('trans')
  u'Privet, Mir!'

Читайте документацию для того, чтобы узнать больше.
Please read the documentation for 

"""

__version__ = '1.3'
__author__ = 'Zelenyak Aleksandr aka ZZZ <zzz.sochi@gmail.com>'

latin = {
    u'à': u'a',  u'á': u'a',  u'â': u'a', u'ã': u'a', u'ä': u'a', u'å': u'a',
    u'æ': u'ae', u'ç': u'c',  u'è': u'e', u'é': u'e', u'ê': u'e', u'ë': u'e',
    u'ì': u'i',  u'í': u'i',  u'î': u'i', u'ï': u'i', u'ð': u'd', u'ñ': u'n',
    u'ò': u'o',  u'ó': u'o',  u'ô': u'o', u'õ': u'o', u'ö': u'o', u'ő': u'o',
    u'ø': u'o',  u'ù': u'u',  u'ú': u'u', u'û': u'u', u'ü': u'u', u'ű': u'u',
    u'ý': u'y',  u'þ': u'th', u'ÿ': u'y',

    u'À': u'A',  u'Á': u'A',  u'Â': u'A', u'Ã': u'A', u'Ä': u'A', u'Å': u'A',
    u'Æ': u'AE', u'Ç': u'C',  u'È': u'E', u'É': u'E', u'Ê': u'E', u'Ë': u'E',
    u'Ì': u'I',  u'Í': u'I',  u'Î': u'I', u'Ï': u'I', u'Ð': u'D', u'Ñ': u'N',
    u'Ò': u'O',  u'Ó': u'O',  u'Ô': u'O', u'Õ': u'O', u'Ö': u'O', u'Ő': u'O',
    u'Ø': u'O',  u'Ù': u'U',  u'Ú': u'U', u'Û': u'U', u'Ü': u'U', u'Ű': u'U',
    u'Ý': u'Y',  u'Þ': u'TH', u'ß': u'ss'
}

greek = {
    u'α': u'a', u'β': u'b', u'γ': u'g', u'δ': u'd', u'ε': u'e',  u'ζ': u'z',
    u'η': u'h', u'θ': u'8', u'ι': u'i', u'κ': u'k', u'λ': u'l',  u'μ': u'm',
    u'ν': u'n', u'ξ': u'3', u'ο': u'o', u'π': u'p', u'ρ': u'r',  u'σ': u's',
    u'τ': u't', u'υ': u'y', u'φ': u'f', u'χ': u'x', u'ψ': u'ps', u'ω': u'w',
    u'ά': u'a', u'έ': u'e', u'ί': u'i', u'ό': u'o', u'ύ': u'y',  u'ή': u'h',
    u'ώ': u'w', u'ς': u's', u'ϊ': u'i', u'ΰ': u'y', u'ϋ': u'y',  u'ΐ': u'i',

    u'Α': u'A', u'Β': u'B', u'Γ': u'G', u'Δ': u'D', u'Ε': u'E',  u'Ζ': u'Z',
    u'Η': u'H', u'Θ': u'8', u'Ι': u'I', u'Κ': u'K', u'Λ': u'L',  u'Μ': u'M',
    u'Ν': u'N', u'Ξ': u'3', u'Ο': u'O', u'Π': u'P', u'Ρ': u'R',  u'Σ': u'S',
    u'Τ': u'T', u'Υ': u'Y', u'Φ': u'F', u'Χ': u'X', u'Ψ': u'PS', u'Ω': u'W',
    u'Ά': u'A', u'Έ': u'E', u'Ί': u'I', u'Ό': u'O', u'Ύ': u'Y',  u'Ή': u'H',
    u'Ώ': u'W', u'Ϊ': u'I', u'Ϋ': u'Y'
}

turkish = {
    u'ş': u's', u'Ş': u'S', u'ı': u'i', u'İ': u'I', u'ç': u'c', u'Ç': u'C',
    u'ü': u'u', u'Ü': u'U', u'ö': u'o', u'Ö': u'O', u'ğ': u'g', u'Ğ': u'G'
}

russian = (
    {u'юй': u'yuy', u'ей': u'yay',
     u'Юй': u'Yuy', u'Ей': u'Yay'},
    {
    u'а': u'a',  u'б': u'b',  u'в': u'v',  u'г': u'g', u'д': u'd', u'е': u'e',
    u'ё': u'yo', u'ж': u'zh', u'з': u'z',  u'и': u'i', u'й': u'y', u'к': u'k',
    u'л': u'l',  u'м': u'm',  u'н': u'n',  u'о': u'o', u'п': u'p', u'р': u'r',
    u'с': u's',  u'т': u't',  u'у': u'u',  u'ф': u'f', u'х': u'h', u'ц': u'c',
    u'ч': u'ch', u'ш': u'sh', u'щ': u'sh', u'ъ': u'',  u'ы': u'y', u'ь': u'',
    u'э': u'e',  u'ю': u'yu', u'я': u'ya',

    u'А': u'A',  u'Б': u'B',  u'В': u'V',  u'Г': u'G', u'Д': u'D', u'Е': u'E',
    u'Ё': u'Yo', u'Ж': u'Zh', u'З': u'Z',  u'И': u'I', u'Й': u'Y', u'К': u'K',
    u'Л': u'L',  u'М': u'M',  u'Н': u'N',  u'О': u'O', u'П': u'P', u'Р': u'R',
    u'С': u'S',  u'Т': u'T',  u'У': u'U',  u'Ф': u'F', u'Х': u'H', u'Ц': u'C',
    u'Ч': u'Ch', u'Ш': u'Sh', u'Щ': u'Sh', u'Ъ': u'',  u'Ы': u'Y', u'Ь': u'',
    u'Э': u'E',  u'Ю': u'Yu', u'Я': u'Ya'
})

ukrainian = (russian[0].copy(), {
    u'Є': u'Ye', u'І': u'I', u'Ї': u'Yi', u'Ґ': u'G',
    u'є': u'ye', u'і': u'i', u'ї': u'yi', u'ґ': u'g'
})
ukrainian[1].update(russian[1])

czech = {
    u'č': u'c', u'ď': u'd', u'ě': u'e', u'ň': u'n', u'ř': u'r', u'š': u's',
    u'ť': u't', u'ů': u'u', u'ž': u'z',
    u'Č': u'C', u'Ď': u'D', u'Ě': u'E', u'Ň': u'N', u'Ř': u'R', u'Š': u'S',
    u'Ť': u'T', u'Ů': u'U', u'Ž': u'Z'
}

polish = {
    u'ą': u'a', u'ć': u'c', u'ę': u'e', u'ł': u'l', u'ń': u'n', u'ó': u'o',
    u'ś': u's', u'ź': u'z', u'ż': u'z',
    u'Ą': u'A', u'Ć': u'C', u'Ę': u'e', u'Ł': u'L', u'Ń': u'N', u'Ó': u'o',
    u'Ś': u'S', u'Ź': u'Z', u'Ż': u'Z'
}

latvian = {
    u'ā': u'a', u'č': u'c', u'ē': u'e', u'ģ': u'g', u'ī': u'i', u'ķ': u'k',
    u'ļ': u'l', u'ņ': u'n', u'š': u's', u'ū': u'u', u'ž': u'z',
    u'Ā': u'A', u'Č': u'C', u'Ē': u'E', u'Ģ': u'G', u'Ī': u'i', u'Ķ': u'k',
    u'Ļ': u'L', u'Ņ': u'N', u'Š': u'S', u'Ū': u'u', u'Ž': u'Z'
}



ascii_str = u'''_0123456789
abcdefghijklmnopqrstuvwxyz
ABCDEFGHIJKLMNOPQRSTUVWXYZ
!"#$%&'()*+,_-./:;<=>?@[\\]^`{|}~ \t\n\r\x0b\x0c'''

ascii = ({}, dict(zip(ascii_str, ascii_str)))
for t in [latin, greek, turkish, russian, ukrainian, czech, polish, latvian]:
    if isinstance(t, dict):
        t = ({}, t)
    ascii[0].update(t[0])
    ascii[1].update(t[1])
ascii[1][None] = u'_'
del t


slug = (ascii[0].copy(), ascii[1].copy())
for c in '''!"#$%&'()*+,_-./:;<=>?@[\\]^`{|}~ \t\n\r\x0b\x0c''':
    del slug[1][c]


def trans(input, table=ascii):
    '''Translate unicode string, using 'table'.
       Table may be tuple (diphthongs, other) or dict (other).'''
    if not isinstance(input, unicode):
        raise TypeError, 'trans codec support only unicode string, %r given.' \
                                                        % type(input)
    if isinstance(table, dict):
        table = ({}, table)

    first = input
    for diphthong, value in table[0].items():
        first = first.replace(diphthong, value)

    default = table[1].get(None, u'_')
    
    second = u''
    for char in first:
        second += table[1].get(char, default)

    return second, len(second)

tables = {'ascii': ascii, 'text': ascii, 'slug': slug, 'id': slug}

import codecs

def encode(input, errors='strict', table_name='ascii'):
    try:
        table = tables[table_name]
    except KeyError:
        raise ValueError, 'Table "%s" not found in tables!' % table_name
    else:
        return trans(input, table)

def no_decode(input, errors='strict'):
    raise TypeError("trans codec does not support decode.")

def trans_codec(enc):
    if enc == 'trans':
        return codecs.CodecInfo(encode, no_decode)
#    else:
    try:
        enc_name, table_name = enc.split('/', 1)
    except ValueError:
        return None
    if enc_name != 'trans':
        return None
    if table_name not in tables:
        raise ValueError, 'Table "%s" not found in tables!' % table_name

    return codecs.CodecInfo(lambda i, e='strict': encode(i, e, table_name),
                            no_decode)

codecs.register(trans_codec)




# -*- coding: utf-8 -*-
'''run on win: py2exe_setup py2exe'''
from distutils.core import setup
import py2exe

setup(
    windows=[{"script":"zenpad.py"}], 
    options={"py2exe": {"includes":["sip"]}}
)